import { KeyPair } from 'waves-crypto';
export declare class WavesConsoleAPI {
    static env: any;
    [key: string]: any;
    static setEnv(env: any): void;
    constructor();
    file: (tabName: string) => string;
    contract: () => string;
    keyPair: (seed?: string | undefined) => KeyPair;
    publicKey: (seed?: string | undefined) => string;
    privateKey: (seed: string) => string;
    address: (seed?: string | undefined, chainId?: string | undefined) => string;
    compile: (code: string) => string;
    broadcast: (tx: any, apiBase?: string | undefined) => Promise<any>;
    deploy: (params?: {
        fee?: number | undefined;
        senderPublicKey?: string | undefined;
        script?: string | undefined;
    } | undefined, seed?: string | string[] | undefined) => Promise<any>;
    private bufferToBase64;
    help: (func?: Function | undefined) => string;
}
/**
 * Item for commands list
 *
 * @interface WavesConsoleAPIHelpCommand
 */
interface WavesConsoleAPIHelpCommand {
    readonly summary?: string;
    readonly description?: string;
    readonly params?: Array<string>;
}
/**
 * Item for variables types list
 *
 * @interface WavesConsoleAPIHelpVariable
 */
interface WavesConsoleAPIHelpVariable {
    readonly optional?: boolean;
    readonly type?: string;
    readonly summary?: string;
}
/**
 * Item for common texts pieces (headers, etc)
 *
 * @interface WavesConsoleApiHelpCommon
 */
interface WavesConsoleApiHelpCommon {
    readonly header?: string;
    readonly summary?: string;
}
/**
 * Help parsers and text compilers
 *
 * @static
 * @class WavesConsoleAPIHelp
 */
export declare class WavesConsoleAPIHelp {
    /**
     * Common texts pieces
     *
     * @static
     * @member {object} common
     */
    static common: {
        [key: string]: WavesConsoleApiHelpCommon;
    };
    /**
     * Commands descriptions vocabulary
     *
     * @static
     * @member {object} texts
     */
    static texts: {
        [key: string]: WavesConsoleAPIHelpCommand;
    };
    /**
     * Variables types vocabulary
     *
     * @static
     * @member {object} types
     */
    static types: {
        [key: string]: WavesConsoleAPIHelpVariable;
    };
    /**
     * Generates API method argument(s) whole description
     *
     * @static
     * @method compileText
     *
     * @param {Array} aliases
     *
     * @returns {string}
     */
    static compileText(aliases: Array<string>): string;
    /**
     * Generates API method arguments description
     *
     * @static
     * @method compileTextArguments
     *
     * @param {Array} args
     * @param {string} text
     *
     * @returns {string}
     */
    static compileTextArguments(args: Array<string>, text: string): string;
    /**
     * Generates API specified method description
     *
     * @static
     * @method compileTextSlice
     *
     * @param {string} alias
     * @param {boolean} full
     * @param {string} text
     *
     * @returns {string}
     */
    static compileTextSlice(alias: string, full: boolean, text: string): string;
}
export {};
