"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const PropTypes = require("prop-types");
const classnames_1 = require("classnames");
const Console_1 = require("./Console");
const Input_1 = require("../containers/Input");
const run_1 = require("../lib/run");
const internal_commands_1 = require("../lib/internal-commands");
const contextBinding_1 = require("../lib/contextBinding");
// this is lame, but it's a list of key.code that do stuff in the input that we _want_.
const doStuffKeys = /^(Digit|Key|Num|Period|Semi|Comma|Slash|IntlBackslash|Backspace|Delete|Enter)/;
class App extends React.Component {
    constructor(props) {
        super(props);
        this.onRun = this.onRun.bind(this);
        this.triggerFocus = this.triggerFocus.bind(this);
        window.sccc = this.scrollToBottom.bind(this);
    }
    onRun(command) {
        return __awaiter(this, void 0, void 0, function* () {
            const console = this.console;
            if (command[0] !== ':') {
                console.push({
                    type: 'command',
                    command,
                    value: command,
                });
                const res = yield run_1.default(command);
                console.push(Object.assign({ command, type: 'response' }, res));
                return;
            }
            let [cmd, ...args] = command.slice(1).split(' ');
            if (/^\d+$/.test(cmd)) {
                args = [parseInt(cmd, 10)];
                cmd = 'history';
            }
            if (!internal_commands_1.default[cmd]) {
                console.push({
                    command,
                    error: true,
                    value: new Error(`No such jsconsole command "${command}"`),
                    type: 'response',
                });
                return;
            }
            let res = yield internal_commands_1.default[cmd]({ args, console, app: this });
            if (typeof res === 'string') {
                res = { value: res };
            }
            if (res !== undefined) {
                console.push(Object.assign({ command, type: 'log' }, res));
            }
            return;
        });
    }
    componentDidMount() {
        run_1.createContainer();
        run_1.bindConsole(this.console);
        contextBinding_1.bindAPItoIFrame(this.props.api);
        const query = decodeURIComponent(window.location.search.substr(1));
        if (query) {
            this.onRun(query);
        }
        else {
            this.onRun(':welcome');
        }
        this.scrollToBottom();
    }
    triggerFocus(e) {
        if (e.target.nodeName === 'INPUT')
            return;
        if (e.metaKey || e.ctrlKey || e.altKey)
            return;
        if (e.code && !doStuffKeys.test(e.code))
            return;
        //this.input.focus();
    }
    scrollToBottom() {
        if (!this.messagesEnd)
            return;
        this.messagesEnd.scrollIntoView({ behavior: "smooth" });
    }
    render() {
        const { commands = [], theme, layout } = this.props;
        const className = classnames_1.default(['App', `theme-${theme}`, layout]);
        return (React.createElement("div", { tabIndex: -1, onKeyDown: this.triggerFocus, ref: e => (this.app = e), className: className },
            React.createElement(Console_1.Console, { ref: e => (this.console = e), commands: commands, reverse: layout === 'top', scrollToBottom: () => this.scrollToBottom() }),
            React.createElement(Input_1.Input, { inputRef: (e) => (this.input = e), onRun: this.onRun, autoFocus: window.top === window, onClear: () => {
                    this.console.clear();
                }, theme: this.props.theme }),
            React.createElement("div", { ref: el => {
                    this.messagesEnd = el;
                } })));
    }
}
App.contextTypes = { store: PropTypes.object };
exports.App = App;
