"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const Line_1 = require("./Line");
let guid = 0;
const getNext = () => guid++;
// function AssertError(message) {
//     this.name = 'Assertion fail';
//     this.message = message;
//     this.stack = new Error().stack;
// }
class AssertError extends Error {
    constructor(message) {
        super(message);
        this.name = 'Assertion fail';
    }
}
// AssertError.prototype = new Error();
function interpolate(...args) {
    let [string, ...rest] = args;
    let html = false;
    if (typeof string === 'string' && string.includes('%') && rest.length) {
        string = string.replace(/(%[scdif]|%(\d*)\.(\d*)[dif])/g, (all, key, width = '', dp) => {
            // NOTE: not supporting Object type
            if (key === '%s') {
                // string
                return rest.shift();
            }
            if (key === '%c') {
                html = true;
                return `</span><span style="${rest.shift()}">`;
            }
            const value = rest.shift();
            let res = null;
            if (key.substr(-1) === 'f') {
                if (isNaN(parseInt(dp, 10))) {
                    res = value;
                }
                else {
                    res = value.toFixed(dp);
                }
            }
            else {
                res = parseInt(value, 10);
            }
            if (width === '') {
                return res;
            }
            return res.toString().padStart(width, ' ');
        });
        if (html) {
            string = `<span>${string}</span>`;
        }
        args = [string, ...rest];
    }
    return { html, args };
}
class Console extends React.Component {
    constructor(props) {
        super(props);
        this.state = (props.commands || []).reduce((acc, curr) => {
            acc.commands[getNext()] = curr;
            return acc;
        }, { commands: {} });
        this.log = this.log.bind(this);
        this.clear = this.clear.bind(this);
        this.push = this.push.bind(this);
        this.scrollToBottom = props.scrollToBottom;
        window.riderepl = this;
    }
    push(command) {
        const next = getNext();
        this.setState({ commands: Object.assign({}, this.state.commands, { [next]: command }) });
    }
    clear() {
        this.setState({ commands: {} });
    }
    error(...rest) {
        const { html, args } = interpolate(...rest);
        this.push({
            error: true,
            html,
            value: args,
            type: 'log',
        });
    }
    ;
    componentDidUpdate() {
        this.scrollToBottom();
    }
    assert(test, ...rest) {
        // intentional loose assertion test - matches devtools
        if (!test) {
            let msg = rest.shift();
            if (msg === undefined) {
                msg = 'console.assert';
            }
            rest.unshift(new AssertError(msg));
            this.push({
                error: true,
                value: rest,
                type: 'log',
            });
        }
    }
    dir(...rest) {
        const { html, args } = interpolate(...rest);
        this.push({
            value: args,
            html,
            open: true,
            type: 'log',
        });
    }
    ;
    warn(...rest) {
        const { html, args } = interpolate(...rest);
        this.push({
            error: true,
            level: 'warn',
            html,
            value: args,
            type: 'log',
        });
    }
    debug(...args) {
        return this.log(...args);
    }
    ;
    info(...args) {
        return this.log(...args);
    }
    ;
    log(...rest) {
        const { html, args } = interpolate(...rest);
        this.push({
            value: args,
            html,
            type: 'log',
        });
    }
    render() {
        const commands = this.state.commands || {};
        const keys = Object.keys(commands);
        if (this.props.reverse) {
            keys.reverse();
        }
        return (React.createElement("div", { className: "react-console-container", onClick: e => {
                e.stopPropagation(); // prevent the focus on the input element
            } }, keys.map(_ => React.createElement(Line_1.Line, Object.assign({ key: `line-${_}` }, commands[_])))));
    }
}
exports.Console = Console;
