"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
// TODO import Autocomplete from './Autocomplete';
const keycodes_1 = require("../lib/keycodes");
class Input extends React.Component {
    constructor(props) {
        super(props);
        // history is set in the componentDidMount
        this.state = {
            value: props.value || '',
            multiline: false,
            rows: 1,
            historyCursor: props.history.length,
        };
        this.onChange = this.onChange.bind(this);
        this.onKeyPress = this.onKeyPress.bind(this);
    }
    onChange() {
        if (!this.input)
            return;
        const { value } = this.input;
        const length = value.split('\n').length;
        this.setState({
            multiline: length > 1,
            rows: length < 20 ? length : 20,
            value,
        });
    }
    onKeyPress(e) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.input)
                return;
            const code = keycodes_1.default[e.keyCode];
            const { multiline } = this.state;
            const { history } = this.props;
            let { historyCursor } = this.state;
            // FIXME in multiline, cursor up when we're at the top
            // const cursor = getCursor(this.input);
            if (e.ctrlKey && code === 'l') {
                this.props.onClear();
                return;
            }
            if (!multiline) {
                if (code === 'up arrow') {
                    historyCursor--;
                    if (historyCursor < 0) {
                        this.setState({ historyCursor: 0 });
                        return;
                    }
                    this.setState({ historyCursor, value: history[historyCursor] });
                    // this.onChange();
                    e.preventDefault();
                    return;
                }
                if (code === 'down arrow') {
                    historyCursor++;
                    if (historyCursor >= history.length) {
                        this.setState({ historyCursor: history.length, value: '' });
                        return;
                    }
                    this.setState({ historyCursor, value: history[historyCursor] });
                    e.preventDefault();
                    return;
                }
            }
            const command = this.input.value;
            if (code === 'enter') {
                if (e.shiftKey) {
                    return;
                }
                if (!command) {
                    e.preventDefault();
                    return;
                }
                this.props.addHistory(command);
                this.setState({ historyCursor: history.length + 1, value: '' });
                e.preventDefault();
                yield this.props.onRun(command);
                // Don't use `this.input.scrollIntoView();` as it messes with iframes
                //window.scrollTo(0, document.body.scrollHeight);
                return;
            }
        });
    }
    render() {
        const { autoFocus } = this.props;
        return (React.createElement("div", { className: "Input" },
            React.createElement("textarea", { className: "cli", rows: this.state.rows, autoFocus: autoFocus, ref: e => {
                    this.input = e;
                    this.props.inputRef(e);
                }, value: this.state.value, onChange: this.onChange, onKeyDown: this.onKeyPress })));
    }
}
exports.Input = Input;
