"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const LineNav_1 = require("./LineNav");
const which_type_1 = require("../lib/which-type");
class Line extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            filter: null,
        };
    }
    shouldComponentUpdate(nextProps, nextState) {
        return this.state.filter !== nextState.filter;
    }
    render() {
        const { type = 'response', value, command = null, error = false, open = false, html = false, onFocus = () => {
        }, } = this.props;
        let line = null;
        const { filter } = this.state;
        if (type === 'command') {
            line = (React.createElement("div", { className: "prompt input" },
                React.createElement(LineNav_1.LineNav, { value: value }),
                value));
        }
        if (type === 'log' || type === 'response') {
            if (type === 'log' && Array.isArray(value) && value.length === 0) {
                return null;
            }
            // for LineNav I do a bit of a giggle so if it's a log, we copy the single
            // value, which is nicer for the user
            line = (React.createElement("div", { className: `prompt output ${type} ${error ? 'error' : ''}` },
                React.createElement(LineNav_1.LineNav, { onFilter: (filter) => {
                        this.setState({ filter });
                    }, value: type === 'log' && Array.isArray(value) && value.length === 1
                        ? value[0]
                        : value, command: command }),
                (type === 'log' && Array.isArray(value) ? value : [value]).map((value, i) => {
                    if (value) {
                        try {
                            const valueType = ({}).toString.call(value);
                            if (valueType === '[object Object]') {
                                value = Object.assign({}, value);
                            }
                        }
                        catch (e) { // only happens when typeof is protected (...randomly)
                        }
                    }
                    const Type = which_type_1.default(value);
                    return (React.createElement(Type, { filter: filter, html: html, value: value, open: open, allowOpen: true, bare: type === 'log', key: `type-${i}`, shallow: false }, value));
                })));
        }
        return (React.createElement("div", { className: "Line", onClick: onFocus }, line));
    }
}
exports.Line = Line;
