"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const Filter_1 = require("./Filter");
const CopyToClipboard = require("react-copy-to-clipboard");
class LineNav extends React.Component {
    constructor(props) {
        super(props);
        this.preCopy = this.preCopy.bind(this);
        this.toggleFilter = this.toggleFilter.bind(this);
        const type = {}.toString.call(props.value) || 'string';
        this.state = {
            text: null,
            type,
            filter: false,
            copyAsHTML: type.includes('Element'),
        };
    }
    preCopy() {
        return __awaiter(this, void 0, void 0, function* () {
            // work out how we should copy this thing
            const original = this.props.value;
            let { value, type } = this.props;
            if (this.state.copyAsHTML) {
                this.setState({ text: value.outerHTML });
                return;
            }
            if (typeof value === 'function') {
                this.setState({ text: value.toString() });
                return;
            }
            if (typeof value === 'string') {
                this.setState({ text: value });
                return;
            }
            if (type === '[object Promise]') {
                const text = yield value;
                this.setState({ text });
                return;
            }
            if (value instanceof Error || type === '[object Error]') {
                // get real props and add the stack no matter what (FF excludes it)
                value = Object.getOwnPropertyNames(value).reduce((acc, curr) => {
                    acc[curr] = value[curr];
                    return acc;
                }, {});
                value.stack = original.stack;
            }
            this.setState({ text: JSON.stringify(value, "", 2) });
        });
    }
    toggleFilter(e) {
        e.preventDefault();
        const filter = !this.state.filter;
        this.setState({
            filter,
        });
    }
    render() {
        const { value, onFilter } = this.props;
        const { text, filter, copyAsHTML } = this.state;
        const copyAs = typeof value === 'function'
            ? 'Copy function'
            : copyAsHTML ? 'Copy as HTML' : 'Copy as JSON';
        return (React.createElement("div", { className: "LineNav" },
            typeof value === 'object' &&
                React.createElement(Filter_1.Filter, { ref: e => (this.filter = e), onFilter: onFilter, enabled: filter },
                    React.createElement("button", { onClick: this.toggleFilter, className: "icon search" }, "search")),
            React.createElement(CopyToClipboard, { text: text },
                React.createElement("button", { title: copyAs, className: "icon copy", onMouseDown: () => {
                        if (text === null) {
                            this.preCopy();
                        }
                    } }, "copy"))));
    }
}
exports.LineNav = LineNav;
