"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const lodash_1 = require("lodash");
const which_type_1 = require("../../lib/which-type");
class ArrayType extends React.Component {
    constructor(props) {
        super(props);
        this.toggle = this.toggle.bind(this);
        this.state = {
            open: props.open,
        };
    }
    toggle(e) {
        if (!this.props.allowOpen) {
            return;
        }
        e.stopPropagation();
        e.preventDefault();
        this.setState({ open: !this.state.open });
    }
    render() {
        const { value, shallow = true, filter = null } = this.props;
        const { open } = this.state;
        let length = value.length;
        if (shallow && !open) {
            return (React.createElement("div", { className: "type ArrayType closed", onClick: this.toggle },
                React.createElement("em", null, "Array"),
                React.createElement("span", { className: "arb-info" },
                    "(",
                    length,
                    ")")));
        }
        let types = value.slice(0, open ? value.length : 10).map((_, i) => {
            const Type = which_type_1.default(_);
            return (React.createElement(Type, { allowOpen: open, key: `arrayType-${i + 1}`, shallow: true, value: _ }, _));
        });
        // expose holes in the collapsed mode
        if (!open) {
            let count = 0;
            const newTypes = [];
            for (let i = 0; i < types.length; i++) {
                const hole = !(i in types);
                if (count !== 0 && !hole) {
                    newTypes.push(React.createElement("span", { key: `hole-${i}`, className: "arb-info" },
                        "<undefined \u00D7 ",
                        count,
                        ">"));
                    count = 0;
                }
                else if (hole) {
                    count++;
                }
                if (!hole) {
                    newTypes.push(types[i]);
                }
            }
            // if there are holes at the end
            if (count !== 0) {
                newTypes.push(React.createElement("span", { key: `hole-${types.length}`, className: "arb-info" },
                    "<undefined \u00D7 ",
                    count,
                    ">"));
            }
            types = newTypes;
        }
        if (!open && value.length > 10) {
            types.push(React.createElement("span", { key: "arrayType-0", className: "more arb-info" }, "\u2026"));
        }
        if (!open) {
            // intersperce with commas
            types = lodash_1.flatten(lodash_1.zip(types, Array.from({ length: types.length - 1 }, (n, i) => (React.createElement("span", { key: `sep-${i}`, className: "sep" }, ",")))));
            // do mini output
            return (React.createElement("div", { className: "type ArrayType closed", onClick: this.toggle },
                React.createElement("em", null, "Array"),
                React.createElement("span", { className: "arb-info" },
                    "(",
                    length,
                    ")"),
                "[ ",
                types,
                " ]"));
        }
        // this is the full output view
        return (React.createElement("div", { className: "type ArrayType" },
            React.createElement("div", { onClick: this.toggle, className: "header" },
                React.createElement("em", null, "Array"),
                React.createElement("span", { className: "arb-info" },
                    "(",
                    length,
                    ")"),
                "["),
            React.createElement("div", { className: "group" }, types.map((type, i) => {
                if (filter === null ||
                    filter === undefined ||
                    filter === '' ||
                    (value[i] + '').toLowerCase().includes(filter)) {
                    return (React.createElement("div", { className: "key-value", key: `subtype-${i}` },
                        React.createElement("span", { className: "index" },
                            i,
                            ":"),
                        type));
                }
                return null;
            })),
            "]"));
    }
}
exports.ArrayType = ArrayType;
