"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const which_type_1 = require("../../lib/which-type");
const StringType_1 = require("./StringType");
const lodash_1 = require("lodash");
const LIMIT_CLOSED = 5;
function* enumerate(obj) {
    let visited = new Set();
    while (obj) {
        for (let key of Reflect.ownKeys(obj)) {
            if (typeof key === 'string') {
                let desc = Reflect.getOwnPropertyDescriptor(obj, key);
                if (desc && !visited.has(key)) {
                    visited.add(key);
                    if (desc.enumerable) {
                        yield key;
                    }
                }
            }
        }
        obj = Reflect.getPrototypeOf(obj);
    }
}
class ObjectType extends React.Component {
    constructor(props) {
        super(props);
        this.toggle = this.toggle.bind(this);
        this.state = {
            open: props.open,
        };
    }
    shouldComponentUpdate(nextProps, nextState) {
        if (this.state.open !== nextState.open) {
            return true;
        }
        if (this.props.filter === undefined) {
            return false; // this prevents bananas amount of rendering
        }
        if (this.props.filter === nextProps.filter) {
            return false;
        }
        return true;
    }
    toggle(e) {
        if (!this.props.allowOpen) {
            return;
        }
        e.stopPropagation();
        e.preventDefault();
        this.setState({ open: !this.state.open });
    }
    render() {
        const { open } = this.state;
        const { filter = null, value, shallow = true, type = {}.toString.call(value), } = this.props;
        let { displayName } = this.props;
        if (!displayName) {
            displayName = value.constructor ? value.constructor.name : 'Object';
        }
        if (!open && shallow) {
            return (React.createElement("div", { onClick: this.toggle, className: `type ${type}` },
                React.createElement("em", null, displayName)));
        }
        let props = open ? [...enumerate(value)] : Object.keys(value);
        Object.getOwnPropertyNames(value).forEach(prop => {
            if (!props.includes(prop)) {
                props.push(prop);
            }
        });
        if (filter !== null) {
            props = props.filter(prop => {
                if ((prop + '').toLowerCase().includes(filter)) {
                    return true;
                }
                if ((value[prop] + '').toLowerCase().includes(filter)) {
                    return true;
                }
                return false;
            });
        }
        if (!open) {
            props.splice(LIMIT_CLOSED);
        }
        let types = props.sort().map((key, i) => {
            const Type = which_type_1.default(value[key]);
            return {
                key,
                value: (React.createElement(Type, { allowOpen: open, key: `objectType-${i + 1}`, shallow: true, value: value[key] })),
            };
        });
        if (!open && Object.keys(value).length > LIMIT_CLOSED) {
            types.push(React.createElement("span", { key: "objectType-0", className: "more" }, "\u2026"));
        }
        if (!open) {
            if (type === 'error') {
                return (React.createElement("div", { className: `type ${type}` },
                    React.createElement("em", { onClick: this.toggle }, displayName),
                    React.createElement("span", null,
                        '{',
                        " ",
                        React.createElement(StringType_1.StringType, { value: value.message }),
                        " ",
                        '}')));
            }
            if (displayName !== 'Object') {
                // just show the summary
                return (React.createElement("div", { className: `type ${type}` },
                    React.createElement("em", { onClick: this.toggle }, displayName),
                    React.createElement("span", null, '{ … }')));
            }
            // intersperce with commas
            types = lodash_1.flatten(lodash_1.zip(types, Array.from({
                length: types.length - 1,
            }, (n, i) => {
                return React.createElement("span", { key: `sep-${i}`, className: "sep" }, ",");
            })));
            // do mini output
            return (React.createElement("div", { className: "type object closed", onClick: this.toggle },
                React.createElement("em", null, displayName),
                React.createElement("span", null,
                    '{',
                    " "),
                types.map((obj, i) => {
                    if (obj && obj.key && obj.value) {
                        return (React.createElement("span", { className: "object-item key-value", key: `subtype-${i}` },
                            React.createElement("span", { className: "key" },
                                obj.key,
                                ":"),
                            React.createElement("span", { className: "value" }, obj.value)));
                    }
                    return obj;
                }),
                React.createElement("span", null,
                    " ",
                    '}')));
        }
        return (React.createElement("div", { className: `type ${type} ${open ? '' : 'closed'}` },
            React.createElement("div", { className: "header" },
                React.createElement("em", { onClick: this.toggle }, displayName),
                React.createElement("span", null, '{')),
            React.createElement("div", { className: "group" }, types.map((obj, i) => {
                return (React.createElement("div", { className: "object-item key-value", key: `subtype-${i}` },
                    React.createElement("span", { className: "key" },
                        obj.key,
                        ":"),
                    React.createElement("span", { className: "value" }, obj.value)));
            })),
            React.createElement("span", null, '}')));
    }
}
exports.ObjectType = ObjectType;
