"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const which_type_1 = require("../../lib/which-type");
class PromiseType extends React.Component {
    constructor(props) {
        super(props);
        this.toggle = this.toggle.bind(this);
        this.state = {
            open: props.open,
            promiseValue: undefined,
            status: 'pending',
        };
    }
    toggle(e) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.props.allowOpen) {
                return;
            }
            e.stopPropagation();
            e.preventDefault();
            const open = !this.state.open;
            if (open) {
                // update promise value
                const { promiseValue, status } = yield this.updatePromiseState();
                return this.setState({ promiseValue, status, open });
            }
            this.setState({ open });
        });
    }
    updatePromiseState() {
        return __awaiter(this, void 0, void 0, function* () {
            let promiseValue = undefined;
            let status = 'pending';
            const flag = Math.random();
            try {
                promiseValue = yield Promise.race([
                    this.props.value,
                    new Promise(resolve => setTimeout(() => resolve(flag), 10)),
                ]);
                if (promiseValue !== flag) {
                    status = 'resolved';
                }
                else {
                    promiseValue = undefined;
                }
            }
            catch (e) {
                promiseValue = e;
                status = 'rejected';
            }
            return {
                promiseValue,
                status,
            };
        });
    }
    componentDidMount() {
        return __awaiter(this, void 0, void 0, function* () {
            const { promiseValue, status } = yield this.updatePromiseState();
            this.setState({ promiseValue, status });
        });
    }
    render() {
        const { filter } = this.props;
        const { open, promiseValue, status } = this.state;
        const Value = which_type_1.default(promiseValue);
        if (!open) {
            return (React.createElement("div", { onClick: this.toggle, className: "type entry closed" },
                React.createElement("em", null, "Promise"),
                '{ ',
                React.createElement("div", { className: "object-item key-value" },
                    React.createElement("span", { className: "key" }, "[[PromiseStatus]]:"),
                    React.createElement("span", { className: "value" }, status)),
                React.createElement("span", { className: "arb-info" }, ", "),
                React.createElement("div", { className: "object-item key-value" },
                    React.createElement("span", { className: "key" }, "[[PromiseValue]]:"),
                    React.createElement("span", { className: "value" },
                        React.createElement(Value, { filter: filter, shallow: true, allowOpen: open, value: promiseValue }))),
                ' }'));
        }
        return (React.createElement("div", { onClick: this.toggle, className: "type promise" },
            React.createElement("div", { className: "header" },
                React.createElement("em", null, "Promise"),
                React.createElement("span", null, '{')),
            React.createElement("div", { className: "group" },
                React.createElement("div", { className: "object-item key-value" },
                    React.createElement("span", { className: "key" }, "[[PromiseStatus]]:"),
                    React.createElement("span", { className: "value" }, status)),
                React.createElement("div", { className: "object-item key-value" },
                    React.createElement("span", { className: "key" }, "[[PromiseValue]]:"),
                    React.createElement("span", { className: "value" },
                        React.createElement(Value, { filter: filter, shallow: true, allowOpen: open, value: promiseValue })))),
            React.createElement("span", null, '}')));
    }
}
exports.PromiseType = PromiseType;
