"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ArrayType_1 = require("../components/types/ArrayType");
const ObjectType_1 = require("../components/types/ObjectType");
const FunctionType_1 = require("../components/types/FunctionType");
const ErrorType_1 = require("../components/types/ErrorType");
const NullType_1 = require("../components/types/NullType");
const UndefinedType_1 = require("../components/types/UndefinedType");
const NumberType_1 = require("../components/types/NumberType");
const StringType_1 = require("../components/types/StringType");
const BooleanType_1 = require("../components/types/BooleanType");
const SetType_1 = require("../components/types/SetType");
const PromiseType_1 = require("../components/types/PromiseType");
function whichType(value) {
    let type = '[object Object]';
    try {
        type = ({}).toString.call(value);
    }
    catch (e) { // only happens when typeof is protected (...randomly)
    }
    if (type === '[object String]') {
        return StringType_1.StringType;
    }
    if (type === '[object Number]') {
        return NumberType_1.NumberType;
    }
    if (type === '[object Boolean]') {
        return BooleanType_1.BooleanType;
    }
    if (type === '[object Set]' || type === '[object Map]') {
        return SetType_1.SetType;
    }
    if (type === '[object Promise]') {
        return PromiseType_1.PromiseType;
    }
    if (value instanceof Error || type === '[object Error]') {
        return ErrorType_1.ErrorType;
    }
    if (value === undefined) {
        return UndefinedType_1.UndefinedType;
    }
    if (value === null) {
        return NullType_1.NullType;
    }
    if (type === '[object Array]') {
        return ArrayType_1.ArrayType;
    }
    if (type === '[object Function]') {
        return FunctionType_1.FunctionType;
    }
    // TODO DOM nodes, etc.
    // everything is eventually an object!
    return ObjectType_1.ObjectType;
}
exports.default = whichType;
