"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("./reducers/index");
const redux_1 = require("redux");
const Settings_1 = require("./actions/Settings");
const Input_1 = require("./actions/Input");
const Env_1 = require("./actions/Env");
const contextBinding_1 = require("./lib/contextBinding");
const save = (key, value, store = 'session') => {
    try {
        const storage = store === 'session' ? window.sessionStorage : window.localStorage;
        storage.setItem(`jsconsole.${key}`, JSON.stringify(value));
    }
    catch (e) {
    }
};
const saveToStorageMiddleware = redux_1.applyMiddleware(store => next => action => {
    const nextAction = next(action);
    const state = store.getState(); // new state after action was applied
    if (action.type === Settings_1.SET_THEME || action.type === Settings_1.SET_LAYOUT) {
        save('settings', state.settings, 'local');
    }
    if (action.type === Input_1.ADD_HISTORY) {
        save('history', state.history);
    }
    if (action.type === Env_1.SET_ENV) {
        save('env', state.env);
    }
    return nextAction;
});
const syncIFrameEnvMiddleware = redux_1.applyMiddleware(store => next => action => {
    const nextAction = next(action);
    const state = store.getState(); // new state after action was applied
    if (action.type === Env_1.SET_ENV) {
        contextBinding_1.updateIFrameEnv(state.env);
    }
    return nextAction;
});
const middlewares = [
    saveToStorageMiddleware,
    syncIFrameEnvMiddleware
];
if (window.__REDUX_DEVTOOLS_EXTENSION__) {
    middlewares.push(window.__REDUX_DEVTOOLS_EXTENSION__());
}
//const finalCreateStore = compose(...middlewares)(createStore);
const defaults = {};
try {
    defaults.settings = JSON.parse(localStorage.getItem('jsconsole.settings') || '{}');
    defaults.history = JSON.parse(sessionStorage.getItem('jsconsole.history') || '[]');
    defaults.env = JSON.parse(sessionStorage.getItem('jsconsole.env') || '{}');
}
catch (e) {
    console.log(e);
}
const store = redux_1.createStore(index_1.default, redux_1.compose(...middlewares));
//const store = finalCreateStore(reducers, defaults);
exports.default = store;
